;****  Ansteuerung eines MAX5154 via SPI  ********************************
;*
;* Ein AT90S8515 gibt via SPI-Schnittstelle ber DACa des angeschlossenen
;* MAX5154 eine ansteigende und ber DACb eine abfallende Spannungsrampe 
;* aus.
;* Taktfrequenz des AVR: 4 MHz (STK200-Standard).
;*  
;*************************************************************************

.nolist
.include "8515def.inc"
.list

.equ  DACa  = $20	; Input-Reg. A laden, DACa-Reg. unverndert
.equ  DACb  = $C0	; Input-Reg. B laden, beide DAC-Reg. aktual.
.equ  nss   = PB4	; /SS = PB4

.def  OutL  = r12	; Low-Byte der SPI-Ausgabe
.def  OutH  = r13	; High-Byte der SPI-Ausgabe
.def  InL   = r14	; Low-Byte der SPI-Eingabe
.def  InH   = r15	; High-Byte der SPI-Eingabe
.def  temp  = r16	; Arbeitsregister
.def  A_lo  = r24	; Low-Byte von DAC_A
.def  A_hi  = r25	; High-Byte Von DAC_A
.def  B_lo  = r26	; Low-Byte von DAC_B
.def  B_hi  = r27	; High-Byte Von DAC_B

Reset:
rjmp  Initial

Output_SPI:	; 16 Bit via SPI, CPHA=0,CPOL=0
cbi   PortB,nss	; /SS = aktiv (log.0)
out   SPDR,OutH	; High-Byte ausgeben
spi1:
sbis  SPSR,SPIF	; Skip, wenn bertragung beendet
rjmp  spi1
in    InH,SPDR	; eingelesenes Byte abspeichern
out   SPDR,OutL	; Low-Byte ausgeben
spi2:
sbis  SPSR,SPIF	; Skip, wenn bertragung beendet
rjmp  spi2
in    InL,SPDR	; eingelesenes Byte abspeichern
sbi   PortB,nss	; /SS = inaktiv (log.1)
ret

Initial:	; Initialisierungsteil
ldi   temp, Low(RAMEND)
out   SPL, Temp
ldi   temp, High(RAMEND)
out   SPH, Temp	; Stackpointer setzen
ldi   temp,$70	; SCK=0,MISO=PullUp,MOSI=1,/SS=1,Rest=HiZ
out   PortB,temp	; Ruhezustand ausgeben 
ldi   temp,$B0	; SCK,MOSI,/SS = Output, Rest = Input
out   DDRB,temp	; Datenrichtung PortB konfigurieren
ldi   temp,$50	; SPIE=0,SPE=1,DORD=0,MSTR=1,CPHA=0,CPOL=0
out   SPCR,temp	;   f(SCK)=f(Sys) / 4
ldi   A_lo,$00
ldi   A_hi,$00	; ZhlerA mit 0 initialisieren
ldi   B_lo,$00
ldi   B_hi,$00	; ZhlerB mit 0 initialisieren
Loop:
adiw  A_lo,2	; Doppelreg. A um 1 inkr. (2 wegen S0=0)
cbr   A_hi,$E0	; Bits 15..13 lschen (berlauf)
sbr   A_hi,DACa	; Steuerbits fr DACa setzen
mov   OutL,A_lo
mov   OutH,A_hi
rcall Output_SPI	; 16 Bit an DACa ber SPI ausgeben
sbiw  B_lo,2	; Doppelreg. B um 1 dekr. (2 wegen S0=0)
cbr   B_hi,$E0	; Bits 15..13 lschen (berlauf)
sbr   B_hi,DACb	; Steuerbits fr DACb setzen
mov   OutL,B_lo
mov   OutH,B_hi
rcall Output_SPI	; 16 Bit an DACb ber SPI ausgeben
rjmp  Loop
