;****  AVR - Porterweiterung via SPI  ************************************
;*
;* Die Anzahl der Ausgangsports eines AT90S4414 wird um 16 Ausgangspins 
;* und 8 Eingangspins erhht.
;* Taktfrequenz des AVR: 4 MHz (STK200-Standard).
;*  
;*************************************************************************
.nolist
.include "4414def.inc"
.list

.def  Last = r15	; Zwischenspeicher
.def  tmp1 = r16	; Arbeitsregister 1
.def  Out1 = r17	; Ausgabewert fr Output 1
.def  Out2 = r18	; Ausgabewert fr Output 2
.def  In1  = r19	; Eingabewert von Input 1

.equ  nss   = PB4	; /SS = PB4
.equ  Latch = PB3	; Latchimpuls = PB3
.equ  okay  = PB1	; LED1 an PortB fr korrekte bertragung
.equ  LEDK  = PB0	; LED0 an PortB fr Taste0
.equ  time  = 117	; T = time * 1024 / 4 MHz = 30 ms

Reset:
rjmp  Initial

WaitTime:	; 30 ms warten
ldi   tmp1,-time	; Timerintervall laden
out   TCNT0,tmp1	; Zhler initialisieren
ldi   tmp1,1<<TOV0	; Bitposition des TOV0-Flags laden
out   TIFR,tmp1	; TOV0-Flag rcksetzen
WT1:
in    tmp1,TIFR	; Timer Interrupt Flag Register holen
sbrs  tmp1,TOV0	; Skip, wenn T/C0-Overflow aufgetreten
rjmp  WT1	; weiter auf Overflow warten
ret

WaitKey:	; auf Taste 0 warten
sbic  PinD,0	; Skip, wenn PD0=0 (Taste gedrckt)
rjmp  WaitKey	; weiter warten
rcall WaitTime	; 30 ms warten (Tastenentprellung)
sbic  PinD,0	; Skip, wenn Taste noch gedrckt
rjmp  WaitKey	; weiter warten
WK2:
sbis  PinD,0	; Skip, wenn PD0=1 (Taste losgelassen)
rjmp  WK2	; weiter warten
rcall WaitTime	; 30 ms warten (Tastenentprellung)
sbis  PinD,0	; Skip, wenn Taste noch losgelassen
rjmp  WK2	; weiter warten
ret

ToggleLED:	; LED0 umschalten
sbis  DDRB,LEDK	; Skip, wenn LED fr Taste ein
rjmp  TL1
cbi   DDRB,LEDK	; LED ausschalten
ret
TL1:
sbi   DDRB,LEDK	; LED einschalten
ret  

SPITransf:	; 16 Bit via SPI, CPHA=0,CPOL=0
cbi   PortB,nss	; /SS = aktiv (log.0)
cbi   PortB,Latch	; Latchimpuls (log.0)
sbi   PortB,Latch	;   ausgeben
out   SPDR,Out2	; Out2 ausgeben
spi1:
sbis  SPSR,SPIF	; Skip, wenn bertragung beendet
rjmp  spi1
in    In1,SPDR	; Input1 einlesen
out   SPDR,Out1	; Out1 ausgeben
spi2:
sbis  SPSR,SPIF	; Skip, wenn bertragung beendet
rjmp  spi2
sbi   PortB,nss	; /SS = inaktiv (log.1)
ret

Initial: 
ldi   tmp1,Low(RAMEND)
out   SPL,tmp1
ldi   tmp1,High(RAMEND)
out   SPH,tmp1	; Stackpointer setzen
ldi   tmp1,$7C	; SCK=0,MISO=PullUp,MOSI=1,/SS=1,
out   PortB,tmp1	;   Latch=1,Bit2=PullUp,LED1,LED0=0
ldi   tmp1,$B8	; SCK,MOSI,/SS,Latch,
out   DDRB,tmp1	;   LED1,LED0=aus,MISO,Bit2=Input
ldi   tmp1,$00	; alle Bits lschen
out   PortD,tmp1	; alle Bits = HiZ (Tasten)
out   DDRD,tmp1	; Datenrichtung PortD = Input
ldi   tmp1,$50	; SPIE=0,SPE=1,DORD=0,MSTR=1, 
out   SPCR,tmp1	;   CPHA=0,CPOL=0, f(SCK)=f(Sys) / 4
ldi   tmp1,5	; Vorteiler einstellen
out   TCCR0,tmp1	; Timer starten, Vorteiler = 1024
clr   Out2	; Ausgabewert initialisieren

Haupt:
rcall WaitKey	; auf Taste 0 warten
rcall ToggleLED	; LED0 umschalten
inc   Out2	; Ausgabewert inkrementieren
mov   Out1,Out2	; nach Out1 kopieren
com   Out1	; 1er-Komplement
rcall SPITransf	; SPI-Transfer Out1, Out2 und In1
sbi   DDRB,okay	; LED1 fr korrekte bertragung ein
cpse  In1,Last	; Input- und letztes Output-Byte vergl.
cbi   DDRB,okay	; LED1 ausschalten, wenn ungleich
mov   Last,Out2	; aktuellen Out1-Wert speichern
rjmp  Haupt

